#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>

// Structure pour représenter un pixel
typedef struct {
    uint8_t blue;
    uint8_t green;
    uint8_t red;
} Pixel;

// Fonction pour charger une image BMP
Pixel* chargerImageBMP(const char* nomFichier, int* largeur, int* hauteur) {
    FILE* fichier = fopen(nomFichier, "rb");
    if (!fichier) {
        printf("Impossible de charger l'image.\n");
        return NULL;
    }

    // Lire l'en-tête du fichier BMP
    fseek(fichier, 18, SEEK_SET);
    fread(largeur, sizeof(int), 1, fichier);
    fread(hauteur, sizeof(int), 1, fichier);
    fseek(fichier, 54, SEEK_SET);

    // Allouer la mémoire pour les pixels
    int tailleImage = (*largeur) * (*hauteur);
    Pixel* pixels = (Pixel*)malloc(tailleImage * sizeof(Pixel));

    // Lire les pixels de l'image
    fread(pixels, sizeof(Pixel), tailleImage, fichier);

    fclose(fichier);

    return pixels;
}

// Fonction pour sauvegarder une image BMP
void sauvegarderImageBMP(const char* nomFichier, const Pixel* pixels, int largeur, int hauteur) {
    FILE* fichier = fopen(nomFichier, "wb");
    if (!fichier) {
        printf("Impossible de sauvegarder l'image.\n");
        return;
    }

    // Écrire l'en-tête du fichier BMP
    int tailleImage = largeur * hauteur;
    int tailleFichier = tailleImage * sizeof(Pixel) + 54;
    uint8_t enTete[54] = {
        0x42, 0x4D,                      // Signature du fichier BMP
        tailleFichier & 0xFF, (tailleFichier >> 8) & 0xFF, (tailleFichier >> 16) & 0xFF, (tailleFichier >> 24) & 0xFF,  // Taille du fichier
        0x00, 0x00, 0x00, 0x00,          // Réservé
        0x36, 0x00, 0x00, 0x00,          // Offset des données de l'image
        0x28, 0x00, 0x00, 0x00,          // Taille de l'en-tête (40 bytes)
        largeur & 0xFF, (largeur >> 8) & 0xFF, (largeur >> 16) & 0xFF, (largeur >> 24) & 0xFF,  // Largeur de l'image
        hauteur & 0xFF, (hauteur >> 8) & 0xFF, (hauteur >> 16) & 0xFF, (hauteur >> 24) & 0xFF,  // Hauteur de l'image
        0x01, 0x00,                      // Planes (1)
        0x18, 0x00,                      // Bits par pixel (24 bits, 8 bits pour chaque canal RGB)
        0x00, 0x00, 0x00, 0x00,          // Compression (sans compression)
        0x00, 0x00, 0x00, 0x00,          // Taille des données de l'image (0 pour les images non compressées)
        0x13, 0x0B, 0x00, 0x00,          // Résolution horizontale (2835 pixels/mètre)
        0x13, 0x0B, 0x00, 0x00,          // Résolution verticale (2835 pixels/mètre)
        0x00, 0x00, 0x00, 0x00,          // Nombre de couleurs de la palette (0 pour les images non indexées)
        0x00, 0x00, 0x00, 0x00           // Nombre de couleurs importantes (0 pour toutes les couleurs importantes)
    };
    fwrite(enTete, sizeof(uint8_t), 54, fichier);

    // Écrire les pixels de l'image
    fwrite(pixels, sizeof(Pixel), tailleImage, fichier);

    fclose(fichier);
}

int main() {
    // Charger l'image
    int largeur, hauteur;
    Pixel* image = chargerImageBMP("image.bmp", &largeur, &hauteur);
    if (!image) {
        return 1;
    }

    // Vérifier les dimensions de l'image
    if (largeur != 600 || hauteur != 400) {
        printf("L'image doit avoir une dimension de 600x400 pixels.\n");
        free(image);
        return 1;
    }

    // Créer les régions d'intérêt (ROI) pour les 6 images découpées
    int largeurDecoupee = 200;
    int hauteurDecoupee = 200;

    // Allouer la mémoire pour les images découpées
    Pixel* image1 = (Pixel*)malloc(largeurDecoupee * hauteurDecoupee * sizeof(Pixel));
    Pixel* image2 = (Pixel*)malloc(largeurDecoupee * hauteurDecoupee * sizeof(Pixel));
    Pixel* image3 = (Pixel*)malloc(largeurDecoupee * hauteurDecoupee * sizeof(Pixel));
    Pixel* image4 = (Pixel*)malloc(largeurDecoupee * hauteurDecoupee * sizeof(Pixel));
    Pixel* image5 = (Pixel*)malloc(largeurDecoupee * hauteurDecoupee * sizeof(Pixel));
    Pixel* image6 = (Pixel*)malloc(largeurDecoupee * hauteurDecoupee * sizeof(Pixel));

    // Copier les régions d'intérêt dans les images découpées
    for (int y = 0; y < hauteurDecoupee; y++) {
        for (int x = 0; x < largeurDecoupee; x++) {
            image1[y * largeurDecoupee + x] = image[y * largeur + x];
            image2[y * largeurDecoupee + x] = image[y * largeur + x + largeurDecoupee];
            image3[y * largeurDecoupee + x] = image[y * largeur + x + 2 * largeurDecoupee];
            image4[y * largeurDecoupee + x] = image[(y + hauteurDecoupee) * largeur + x];
            image5[y * largeurDecoupee + x] = image[(y + hauteurDecoupee) * largeur + x + largeurDecoupee];
            image6[y * largeurDecoupee + x] = image[(y + hauteurDecoupee) * largeur + x + 2 * largeurDecoupee];
        }
    }

    // Sauvegarder les images découpées
    sauvegarderImageBMP("1.bmp", image1, largeurDecoupee, hauteurDecoupee);
    sauvegarderImageBMP("2.bmp", image2, largeurDecoupee, hauteurDecoupee);
    sauvegarderImageBMP("3.bmp", image3, largeurDecoupee, hauteurDecoupee);
    sauvegarderImageBMP("4.bmp", image4, largeurDecoupee, hauteurDecoupee);
    sauvegarderImageBMP("5.bmp", image5, largeurDecoupee, hauteurDecoupee);
    sauvegarderImageBMP("6.bmp", image6, largeurDecoupee, hauteurDecoupee);

    // Libérer la mémoire
    free(image);
    free(image1);
    free(image2);
    free(image3);
    free(image4);
    free(image5);
    free(image6);

    printf("Les images ont été découpées avec succès.\n");

    return 0;
}
